#!/usb/bin/python3
import os
import sys
import time
import datetime
from prometheus_client import CollectorRegistry, Gauge, Counter, push_to_gateway

prom_push_gw = os.getenv("PUSH_GW", "prod-cp-prometheus-pushgateway:7071")

registry = CollectorRegistry()
c = Counter('mongo_backup_size', 'Last backup size', registry=registry)
g = Gauge('mongo_backup_job_last_success_time', 'Last time a mongo backup job successfully finished', registry=registry)


def _str_file_st(f: str, st: os.stat_result) -> str:
    return "file: %020s size(MB): %8.2f date: %s" % (f, st.st_size / 1024 / 1024, datetime.datetime.fromtimestamp(st.st_mtime).isoformat())


def manage_backup_files(folder: str, retention_days: int, match_file_subs: str) -> int:
    finfo = {}
    to_remove_files = {}
    last_backup_size = 0
    # por - point of no return
    por = time.time() - (retention_days * 60 * 60 * 24)

    os.chdir(folder)
    for f in os.listdir(folder):
        # check this filename loosely resembles a backup file name
        if f.find(match_file_subs) != -1:
            st = os.stat(f)
            # I guess a backup file should be a little bigger than 1k
            if st.st_size > 1000:
                finfo[f] = st

                # check if this backup file is older than retention days
                if st.st_mtime <= por:
                    to_remove_files[f] = st

                # check who is the last backup - within 24h
                if st.st_mtime >= (time.time() - (24 * 60 * 60)):
                    last_backup_size = st.st_size

    ab = "\n".join([_str_file_st(f, st) for f, st in finfo.items()])
    tr = "\n".join([_str_file_st(f, st) for f, st in to_remove_files.items()])
    print("ALL BACKUPS:\n%s\nTO REMOVE BACKUPS:\n%s" % (ab, tr))

    c.inc(last_backup_size)
    g.set_to_current_time()
    try:
        print("Prometheus GW: %s" % prom_push_gw)
        push_to_gateway(prom_push_gw, job='mongo_backup', registry=registry)
    except Exception as e:
        print("Pushing prometheus metrics failed %s" % str(e))

    if len(finfo) < retention_days:
        print("\n\nNOTHING TO REMOVE. BACKUPS ARE LESS THAN %d" % retention_days)
        return -1

    if len(finfo) - len(to_remove_files) < retention_days:
        # Oh, NO! If we delete the old files we are left with less than required backups
        # print a big fat warning and abort the cleanup.
        print("""
           !!!!! LOOKS LIKE SOMETHING IS TERRIBLY WRONG WITH THE BACKUP SYSTEM !!!!!
        !!!!! IF WE REMOVE THE OLD BACKUPS WE WILL BE LEFT WITH NO ENOUGH BACKUPS !!!!!
        """)
        return -1

    # We have reached the point where we are supposed to delete the older backups
    for f in to_remove_files.keys():
        os.unlink(f)


if __name__ == "__main__":
    if len(sys.argv) != 4:
        print("\nUSAGE: %s [backup_folder] [retention_days] [substring_from_the_backup_file]" % sys.argv[0])
        print("\nEXAMPLE:\n\t%s /data/backup 5 backup" % sys.argv[0])
    else:
        rd = int(sys.argv[2])
        if not (2 <= rd <= 365):
            print("retention_days is not quite sane should be between 2 and 365 days")
            sys.exit(-1)

        sys.exit(
            manage_backup_files(sys.argv[1], int(sys.argv[2]), sys.argv[3])
        )
